#include "manageclients.h"
#include "ui_manageclients.h"
#include <QImage>
#include <QDir>
#include "mainwindow.h"
#include <QStringList>
#include <QMessageBox>
#include <QTime>

#include  <QtConcurrentRun>


ManageClients::ManageClients(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ManageClients)
{
    ui->setupUi(this);


    StopOperation=false;

    //Columns for tableview
    modelTable = new QStandardItemModel();
    modelTable->setHorizontalHeaderItem(0, new QStandardItem(QString(tr("Name"))));
    modelTable->setHorizontalHeaderItem(1, new QStandardItem(QString(tr("IP address"))));
    modelTable->setHorizontalHeaderItem(2, new QStandardItem(QString(tr("Port"))));
    modelTable->setHorizontalHeaderItem(3, new QStandardItem(QString(tr("Password"))));
    modelTable->setHorizontalHeaderItem(4, new QStandardItem(QString(tr("Command"))));

    ui->tableViewClients->setModel(modelTable);
    ui->tableViewClients->setColumnWidth(0,300);
    ui->tableViewClients->verticalHeader()->setVisible(false);
    ui->cmdStopNow->setEnabled(false);


    ReadClientsConfig();
    LoadClientsTable();



}

ManageClients::~ManageClients()
{
    delete ui;
}

void ManageClients::on_cmdClose_clicked()
{
    StopOperation=true;
    this->close();
    this->destroy();
}

void ManageClients::ReadClientsConfig()
{
#ifdef Q_OS_LINUX
    //Folder for settings (.ini file)
    QString settingsHome=QDir::homePath()+"/.config/logout" ;
    if (!QDir(settingsHome).exists()) QDir().mkdir(settingsHome);

    //Settings file (.ini)
    settingsFile=settingsHome+"/clients.ini";
#endif
#ifdef Q_OS_WIN32
    //Folder for settings (.ini file)
    QString settingsHome=QDir::homePath()+"/logout" ;
    if (!QDir(settingsHome).exists()) QDir().mkdir(settingsHome);

    //Settings file (.ini)
    settingsFile=settingsHome+"/clients.ini";
#endif

}

QString ManageClients::GetIpFromKey(QString key)
{

   QSettings settings(settingsFile, QSettings::IniFormat);
   QStringList mkey=settings.value(key).toString().simplified().split(':');
   return mkey[0];
}

QString ManageClients::GetPortFromKey(QString key)
{
    QSettings settings(settingsFile, QSettings::IniFormat);
    QStringList mkey=settings.value(key).toString().simplified().split(':');
    return mkey[1];
}

QString ManageClients::GetPasswordFromKey(QString key)
{
    QSettings settings(settingsFile, QSettings::IniFormat);
    QStringList mkey=settings.value(key).toString().simplified().split(':');
    return mkey[2];
}

QString ManageClients::GetCommandFromKey(QString key)
{
    QSettings settings(settingsFile, QSettings::IniFormat);
    QStringList mkey=settings.value(key).toString().simplified().split(':');
    return mkey[3];

}

void ManageClients::LoadClientsTable()
{
    //First clear all
    modelTable->removeRows(0, modelTable->rowCount());

    QSettings settings(settingsFile, QSettings::IniFormat);
    QStringList Clients= settings.childKeys();


    for (int i=0; i<=Clients.count()-1; i++)
    {

        if (!Clients[i].isNull())
        {
            //Name
            QStandardItem *cellName = new QStandardItem(Clients[i]);
            cellName->setIcon(QIcon(":/icons/emblem-synchronizing.png"));
            modelTable->setItem(i,0,cellName);

            //Ip
            QStandardItem *cellIp = new QStandardItem(GetIpFromKey(Clients[i]));
            modelTable->setItem(i,1,cellIp);

            //Port
            QStandardItem *cellPort = new QStandardItem(GetPortFromKey(Clients[i]));
            modelTable->setItem(i,2,cellPort);

            //Password
            QStandardItem *cellPassword = new QStandardItem(GetPasswordFromKey(Clients[i]));
            modelTable->setItem(i,3,cellPassword);

            //Ip
            QStandardItem *cellCommand = new QStandardItem(GetCommandFromKey(Clients[i]));
            modelTable->setItem(i,4,cellCommand);
        }

    }
}

void ManageClients::ToggleGui()
{
    if (ui->cmdCheckClients->isEnabled())
    {
        ui->cmdCheckClients->setEnabled(false);
        ui->cmdAddClient->setEnabled(false);
        ui->cmdSaveClient->setEnabled(false);
        ui->cmdDeleteClient->setEnabled(false);
        ui->cmdSendToAllClients->setEnabled(false);
        ui->cmdSendToClient->setEnabled(false);
        ui->txtMessageForClient->setEnabled(false);
    }else{
        ui->cmdCheckClients->setEnabled(true);
        ui->cmdAddClient->setEnabled(true);
        ui->cmdSaveClient->setEnabled(true);
        ui->cmdDeleteClient->setEnabled(true);
        ui->cmdSendToAllClients->setEnabled(true);
        ui->cmdSendToClient->setEnabled(true);
        ui->txtMessageForClient->setEnabled(true);
    }
}

void ManageClients::on_cmdStopNow_clicked()
{
    StopOperation=true;
    ui->cmdStopNow->setEnabled(false);

}


void ManageClients::on_cmdSendToClient_clicked()
{
    SendToOne(ServerName);
    ui->tableViewClients->selectRow(indexRow);

}



void ManageClients::CheckAllTableClientsStatus()
{
    LoadClientsTable();
    ui->cmdStopNow->setEnabled(true);
    QSettings settings(settingsFile, QSettings::IniFormat);
    QStringList Clients= settings.childKeys();
    QTcpSocket *socket= new  QTcpSocket(this);

    //progressBar
    int rowCount;
    int PercentNow=0;

    if (modelTable->rowCount()-1<=0)rowCount=1; else rowCount=100/modelTable->rowCount() ;
    int PercentIncrement=100/rowCount;

    ui->progressBar->setMaximum(100);
    ToggleGui();



    for (int i=0; i<=modelTable->rowCount()-1; i++)
    {
        if (StopOperation==true)
        {
            StopOperation=false;
            WriteStatus(tr("Operation canceled by user."));
            break;
        }

        if (Clients[i]!="")
        {
            QApplication::processEvents();
            socket->connectToHost(GetIpFromKey(Clients[i]), GetPortFromKey(Clients[i]).toInt());
            if(socket->waitForConnected(2000))
            {
                socket->write("ping");
                socket->waitForBytesWritten();
                socket->flush();

                WriteInfoLog(Clients[i]+tr(" connected."));
                QStandardItem *cellName = new QStandardItem(Clients[i]);
                cellName->setIcon(QIcon(":/icons/emblem-default.png"));
                modelTable->setItem(i,0,cellName);

            }else{
                WriteErrorLog(Clients[i]+tr(" disconnected."));
                QStandardItem *cellName = new QStandardItem(Clients[i]);
                cellName->setIcon(QIcon(":/icons/dialog-error.png"));
                modelTable->setItem(i,0,cellName);
            }
            socket->close();
            PercentNow=PercentNow+PercentIncrement;
            ui->progressBar->setValue(PercentNow);

            QApplication::processEvents();
        }
    }

    WriteStatus(tr("Operation completed."));
    ui->progressBar->setValue(100);
    ui->tableViewClients->clearSelection();
    QMessageBox::information(0, tr("Information"), tr("All server are scanned."), QMessageBox::Ok);
    ui->progressBar->setValue(0);
    ToggleGui();
    ui->cmdStopNow->setEnabled(false);
    StopOperation=false;

}


void ManageClients::WriteInfoLog(QString txt)
{
    QString date = QDate::currentDate().toString();
    QString time = QTime::currentTime().toString();

    ui->textEditLog->append("<p style=\"color: green\">[INFO] - "+date+"-"+time+" - "+txt+"</p>");
}

void ManageClients::WriteErrorLog(QString txt)
{
    QString date = QDate::currentDate().toString();
    QString time = QTime::currentTime().toString();
    ui->textEditLog->append("<p style=\"color: red\">[ERROR] - "+date+"-"+time+" - "+txt+"</p>");
}

void ManageClients::WriteStatus(QString txt)
{
    QString date = QDate::currentDate().toString();
    QString time = QTime::currentTime().toString();
    ui->textEditLog->append("<p style=\"margin-top:10px; color: #7F7F7F; font-weight: bold\">"+date+"-"+time+" - "+txt+"</p>");
}

void ManageClients::ClearLog()
{
    ui->textEditLog->clear();
    ui->textEditLog->append("<html><head></head><body><span style=\" font-size:14pt; font-weight:600; color:#5b963b;\">Ready!</span></p></body></html>");
}


void ManageClients::SendToOne(QString name)
{

    socket= new  QTcpSocket(this);
    socket->connectToHost(GetIpFromKey(name), GetPortFromKey(name).toInt());
    connect(socket, SIGNAL(readyRead()),this,SLOT(packetToOne()));

    if(socket->waitForConnected(2000))
    {
        //send
        QString d;
        if (!ui->checkBoxOnlyMessage->isChecked())
        {
            d=GetPasswordFromKey(name)+":"+GetCommandFromKey(name)+":"+ui->txtMessageForClient->text();
        }else{
            d=GetPasswordFromKey(name)+"::"+ui->txtMessageForClient->text();
        }

        socket->write(d.toAscii());
        socket->waitForBytesWritten();
        socket->flush();
        WriteInfoLog(tr("Send command in progress to ")+name+"...");

        //Table Icons
        QStandardItem *cellName = new QStandardItem(name);
        cellName->setIcon(QIcon(":/icons/emblem-default.png"));
        modelTable->setItem(indexRow,0,cellName);

    }else{
        QMessageBox::warning(0, tr("Warning"), tr("Server unreachable."), QMessageBox::Ok);
        WriteErrorLog(name+tr(" <b>unreachable.</b>"));

        //Table Icons
        QStandardItem *cellName = new QStandardItem(name);
        cellName->setIcon(QIcon(":/icons/dialog-error.png"));
        modelTable->setItem(indexRow,0,cellName);
    }


}


void ManageClients::packetToOne()
{

    qDebug()<<"PackettoOne()";
    //recv
    QByteArray data;
    socket->waitForReadyRead(2000);
    data=socket->readLine();
    socket->flush();
    qDebug()<<"<SERVER>"<<data;
    if (strcmp(data,"GOOD-PASSWORD")==0)
    {
        //QMessageBox::information(0, tr("Info"), tr("Command sent."), QMessageBox::Ok);
        WriteInfoLog(ServerName+tr(" <b>command accepted.</b>"));
    }else{

        //QMessageBox::warning(0, tr("Warning"), tr("Login failed."), QMessageBox::Ok);
        WriteErrorLog(ServerName+tr(" <b>login failed.</b>"));
    }
     socket->close();

}


ClientsThread::ClientsThread()
{


}

void ClientsThread::run()
{


}



void ManageClients::on_cmdCheckClients_clicked()
{
    //QTimer::singleShot(1000, this, SLOT(CheckAllTableClientsStatus()));
    // QFuture<void> future = QtConcurrent::run(this,&ManageClients::CheckAllTableClientsStatus);

   CheckAllTableClientsStatus();
}

void ManageClients::on_tableViewClients_clicked(const QModelIndex &index)
{
    ui->tableViewClients->selectRow(index.row());
    ServerName=modelTable->item(index.row(),0)->data(Qt::DisplayRole).toString();
    indexRow=index.row();

    qDebug()<<"Selected server name->"<< ServerName;
}



void ManageClients::on_cmdSaveClient_clicked()
{
   QSettings settings(settingsFile, QSettings::IniFormat);
   if (ServerName!="") settings.remove(ServerName);

   for (int i=0; i<=modelTable->rowCount()-1; i++)
   {
       QString Name=modelTable->item(i,0)->data(Qt::DisplayRole).toString();
       QString Ip=modelTable->item(i,1)->data(Qt::DisplayRole).toString();
       QString Port=modelTable->item(i,2)->data(Qt::DisplayRole).toString();
       QString Password=modelTable->item(i,3)->data(Qt::DisplayRole).toString();
       QString Command=modelTable->item(i,4)->data(Qt::DisplayRole).toString();

       Ip.remove(QChar(':'), Qt::CaseInsensitive);
       Port.remove(QChar(':'), Qt::CaseInsensitive);
       Password.remove(QChar(':'), Qt::CaseInsensitive);
       Command.remove(QChar(':'), Qt::CaseInsensitive);

       if (Name!="") settings.setValue(Name,Ip+":"+Port+":"+Password+":"+Command);

   }
   LoadClientsTable();

}


void ManageClients::on_cmdDeleteClient_clicked()
{
    QSettings settings(settingsFile, QSettings::IniFormat);
    settings.remove(ServerName);
    LoadClientsTable();
}


void ManageClients::on_cmdAddClient_clicked()
{
   ServerName="";
   QStandardItem* cellName = new QStandardItem("");
   QStandardItem *cellIp = new QStandardItem("");
   QStandardItem *cellPort = new QStandardItem("");
   QStandardItem *cellPassword = new QStandardItem("");
   QStandardItem *cellCommand = new QStandardItem("");

   QList<QStandardItem*> row;
   row.append(cellName);
   row.append(cellIp);
   row.append(cellPort);
   row.append(cellPassword);
   row.append(cellCommand);

   modelTable->appendRow(row);


}



void ManageClients::on_cmdSendToAllClients_clicked()
{
    ToggleGui();
    ui->cmdStopNow->setEnabled(true);
    StopOperation=false;

    //progressBar
    int rowCount;
    int PercentNow=0;

    if (modelTable->rowCount()-1<=0)rowCount=1; else rowCount=100/modelTable->rowCount() ;
    int PercentIncrement=100/rowCount;
    ui->progressBar->setMaximum(100);

    for (int i=0; i<=modelTable->rowCount()-1; i++)
    {
        if (StopOperation==true)
        {
            WriteStatus(tr("Operation canceled by user"));
            ui->cmdStopNow->setEnabled(false);
            StopOperation=false;
            break;
        }
        QApplication::processEvents();

        QString Name=modelTable->item(i,0)->data(Qt::DisplayRole).toString();
        QString Ip=modelTable->item(i,1)->data(Qt::DisplayRole).toString();
        QString Port=modelTable->item(i,2)->data(Qt::DisplayRole).toString();
        QString Password=modelTable->item(i,3)->data(Qt::DisplayRole).toString();
        QString Command=modelTable->item(i,4)->data(Qt::DisplayRole).toString();
        ServerName=Name;

        socket= new  QTcpSocket(this);
        socket->connectToHost(Ip, Port.toInt());
        connect(socket, SIGNAL(readyRead()),this,SLOT(packetToOne()));

        if(socket->waitForConnected(2000))
        {
            //send
            QString d;
            if (!ui->checkBoxOnlyMessage->isChecked())
            {
                d=Password+":"+Command+":"+ui->txtMessageForClient->text();
            }else{
                d=Password+"::"+ui->txtMessageForClient->text();
            }

            socket->write(d.toAscii());
            socket->waitForBytesWritten();
            socket->flush();
            WriteInfoLog(tr("Send command in progress to ")+Name+"...");

            //Table Icons
            QStandardItem *cellName = new QStandardItem(Name);
            cellName->setIcon(QIcon(":/icons/emblem-default.png"));
            modelTable->setItem(i,0,cellName);

        }else{
            //QMessageBox::warning(0, tr("Warning"), tr("Server unreachable."), QMessageBox::Ok);
            WriteErrorLog(Name+tr(" <b>unreachable.</b>"));

            //Table Icons
            QStandardItem *cellName = new QStandardItem(Name);
            cellName->setIcon(QIcon(":/icons/dialog-error.png"));
            modelTable->setItem(i,0,cellName);
        }
        PercentNow=PercentNow+PercentIncrement;
        ui->progressBar->setValue(PercentNow);
        QApplication::processEvents();
    }
    ui->progressBar->setValue(100);
    ToggleGui();
    ui->cmdStopNow->setEnabled(false);
    StopOperation=false;
    QMessageBox::information(0, tr("Information"), tr("All commands are sent."), QMessageBox::Ok);
    ui->progressBar->setValue(0);
}
